// bind.cpp

#include "bind.h"

const Bind::Type Bind::_types[] = {
    Type("keybind",
            "key combination to be bound, it has the following "\
            "syntax: Modifier-Modifier-Key; any number of modifiers can "\
            "be specified, following each one with a '-', the available "\
            "modifiers are:\n"\
            "* S - the Shift key,\n"\
            "* C - the Control key,\n"\
            "* A - the Alt key,\n"\
            "* W - the Windows keys,\n"\
            "* M - Mod3, not set by default i think;\n"\
            "Mod1 though Mod5 may also be used as modifiers, the "\
            "xmodmap tool is used to set these up; the A and W "\
            "modifiers are merely aliases for Mod1 and Mod4; the Key is "\
            "the name of the key, such as 'a' or 'F1' or '4'; you can "\
            "find the name of a key by running xev in a terminal, "\
            "pressing the key, and watching the output from xev in the "\
            "terminal", 
            false, "key"),
    Type("mousebind",
            "mouse button and keyboard modifiers being bound, the "\
            "button has the following syntax: Modifier-Modifier-Button; "\
            "any number of modifiers can be specified, following each "\
            "one with a '-', the Button can be the button's number, "\
            "such as 1 or 3, or it can be a name; the available names "\
            "are Left, Middle, Right, Up, and Down; also specifies what "\
            "you are doing with your mouse for the binding; it can be "\
            "one of the following:\n"\
            "* Press - a mouse button is pressed,\n"\
            "* Release - a mouse button is released,\n"\
            "* Click - a mouse button is pressed and released,\n"\
            "* DoubleClick - a mouse button is pressed and released "\
            "twice, quickly\n"\
            "  (the time delay in which both clicks must occur is "\
            "configurable\n"\
            "* Drag - a mouse button is pressed, and the mouse is moved "\
            "while holding the button down",
            true, "button")
};

const gchar *Bind::_key_action_names[] =
{ "Press", "Release", "Click", "DoubleClick", "Drag", 0 };

Bind::Bind(xmlNodePtr   node, 
           const gchar *path,
           const gchar *attr_name,
           const gchar *attr_value,
           int          type) :
    _node(xml_find(node, path, true, 0, attr_name, attr_value)),
    _type(type),
    _actions(0),
    _binds(0)
{
    // read sub binds
    xmlNodePtr sub;
    sub = xml_find(_node, _types[_type]._name);
    while(sub) {
        Bind *bind = new Bind(sub, _type);
        _binds = g_list_append(_binds, bind);
        sub = xml_next(sub);
    }
}

Bind::Bind(xmlNodePtr node,
           int type) :
    _node(node),
    _type(type),
    _actions(0),
    _binds(0)
{
    // read sub actions
    xmlNodePtr sub = xml_find(_node, "action");
    Action *act = (sub ? new Action(sub) : 0);
    while(act) {
        _actions = g_list_append(_actions, act);
        act = act->find_next();
    }
    // read sub binds if keyboard binds
    if(!_type) {
        sub = xml_find(_node, _types[_type]._name);
        while(sub) {
            Bind *bind = new Bind(sub, _type);
            _binds = g_list_append(_binds, bind);
            sub = xml_next(sub);
        }
    }
}

Bind::~Bind()
{
    for(GList *c = g_list_first(_actions); c; c = g_list_next(c))
        delete ((Action*)c->data);
    g_list_free(g_list_first(_actions));
    for(GList *c = g_list_first(_binds); c; c = g_list_next(c))
        delete ((Bind*)c->data);
    g_list_free(g_list_first(_binds));
}

void Bind::remove() { xml_remove(_node); }

Action *Bind::first_action() const
{
    _actions = g_list_first(_actions);
    if(_actions)
        return (Action *)_actions->data;
    return 0;
}

Action *Bind::next_action() const
{
    GList *next = g_list_next(_actions);
    if(next) {
        _actions = next;
        return (Action *)next->data;
    }
    return 0;
}

Bind *Bind::first_bind() const
{
    _binds = g_list_first(_binds);
    if(_binds)
        return (Bind *)_binds->data;
    return 0;
}

Bind *Bind::next_bind() const
{
    GList *next = g_list_next(_binds);
    if(next) {
        _binds = next;
        return (Bind *)next->data;
    }
    return 0;
}

Action *Bind::add(int i)
{
    Action *res;
    if(!_actions)
        res = new Action(xml_find(_node, "action", true));
    else {
        _actions = g_list_last(_actions);
        res = ((Action*)(_actions->data))->find_next(true);
    }
    res->set_pattern(i);
    _actions = g_list_append(_actions, res);
    return res;
}

Bind *Bind::add(const gchar *key, int t)
{
    xmlNodePtr sub;
    if(!_binds)
        sub = xml_find(_node, _types[_type]._name, true);
    else {
        _binds = g_list_last(_binds);
        sub = xml_next(((Bind*)(_binds->data))->_node, true);
    }
    Bind *bind = new Bind(sub, _type);
    bind->set_key(key);
    if(_type)
        bind->set_key_action(t);
    _binds = g_list_append(g_list_first(_binds), bind);
    return bind;
}

void Bind::del(Action *to_delete)
{
    to_delete->remove();
    _actions = g_list_remove(_actions, to_delete);
}

void Bind::del(Bind *to_delete)
{
    to_delete->remove();
    _binds = g_list_remove(_binds, to_delete);
}

const gchar *Bind::get_key() const
{
    return xml_attr(_node, _types[_type]._key_name);
}
void Bind::set_key(const gchar *key)
{
    xml_attr(_node, _types[_type]._key_name, key);
}

int Bind::get_key_action() const
{
    const gchar *pc = xml_attr(_node, "action");
    if(_type && pc)
        for(int i = 0; _key_action_names[i]; ++i)
            if(!utf8_cmp(_key_action_names[i], pc))
                return i;
    return 0;
}

void Bind::set_key_action(int i)
{
    xml_attr(_node, "action", _key_action_names[i]);
}

int Bind::type() const 
{ 
    return _type;
}

const gchar *Bind::key_actions_name(int i)
{ 
    return ( (i >= 0 && i < 5) ? _key_action_names[i] : 0 );
}

const gchar *Bind::bind_types_note(int i)
{
    return ( (i >= 0 && i < 2) ? _types[i]._note : 0 );
}
