// xml.cpp

#include "xml.h"
#include <openbox/parse.h>

void xml_save(xmlDocPtr    doc,
              const gchar *path)
{
    if(doc && path) xmlSaveFormatFileEnc(path, doc, "UTF-8", 1);
}

xmlNodePtr xml_find(xmlNodePtr   node,
                    const gchar *path,
                    bool         create,
                    const gchar *def,
                    const gchar *attr_name,
                    const gchar *attr_value)
{
    xmlNodePtr res = node;
    if(!path || !node)
        return res;
    gchar **names = g_strsplit(path, "/", 0);
    gchar **next_name;
    for(gchar **curr_name = names; *curr_name; ++curr_name) {
        xmlNodePtr sub_node = parse_find_node(*curr_name, res->children);
        if(!sub_node && create)
            sub_node = xmlNewTextChild(res, 0, BAD_CAST (*curr_name),
                                       BAD_CAST (*(curr_name + 1) ? 0 : def));
        res = sub_node;
    }
    g_strfreev(names);
    if(attr_name && res) {
        const xmlChar *name = res->name;
        xmlNodePtr parent = res->parent;
        while(res) {
            gchar *value;
            if(parse_attr_contains(attr_value, res, attr_name)) break;
            res = res->next;
        }
        if(!res && create) {
            res = xmlNewTextChild(parent, 0, name, 0);
            xmlSetProp(res, BAD_CAST attr_name, BAD_CAST attr_value);
        }
    }
    if(res) {
        gchar *pc = xml_content(res);
        if((!pc || !pc[0]) && def && create)
            xml_content(res, def);
        if(pc) g_free(pc);
    }
    return res;
}

xmlNodePtr xml_next(xmlNodePtr   node,
                    bool         create,
                    const gchar *def)
{
    xmlNodePtr res = node->next;
    while(res)
        if(!g_utf8_collate((const gchar*) node->name,
                           (const gchar*) res->name))
            return res;
        else
            res = res->next;
    if(create) {
        res = xmlNewChild(node->parent, 0, node->name, 0);
        if(def)
            xml_content(res, def);
    }
    return res;
}

const gchar *xml_attr(xmlNodePtr   node,
                      const gchar *name,
                      const gchar *value)
{
    if(value) {
        xmlSetProp(node, BAD_CAST name, BAD_CAST value);
        return value;
    }
    return (const gchar *) xmlGetProp(node, BAD_CAST name);
}

const gchar *xml_name(xmlNodePtr   node,
                      const gchar *name)
{
    if(name) {
        xmlNodeSetName(node, BAD_CAST name);
        return name;
    }
    else
        return (const gchar *) node->name;
}

gchar *xml_content(xmlNodePtr   node,
                   const gchar *content)
{
    gchar *res = 0;
    if(content)
        xmlNodeSetContent(node, BAD_CAST(content));
    else {
        xmlChar *xc = xmlNodeListGetString(node->doc, node->children, true);
        if(xc) {
            res = g_strdup((const gchar*) xc);
            xmlFree(xc);
        }
    }
    return res;
}

void xml_remove(xmlNodePtr node)
{ 
    xmlUnlinkNode(node);
    xmlFreeNode(node);
}

int utf8_cmp(const gchar *pc1,
             const gchar *pc2)
{
    if(!pc1 && !pc2) return 0;
    else if(!pc1) return -1;
    else if(!pc2) return 1;
    gchar *s1 = g_utf8_casefold(pc1, -1);
    gchar *s2 = g_utf8_casefold(pc2, -1);
    int res = g_utf8_collate(s1, s2);
    g_free(s1);
    g_free(s2);
    return res;
}

